!=======================================================================
! Prepares anelastic coefficients Y at grid line L for integration
! and calculates unrelaxed elastic moduli mu and kappa
!=======================================================================

SUBROUTINE PREPARE_INTEG_Y(LL)

  USE MOD_PRECISION , ONLY : WP
  USE MOD_ATTEN     , ONLY : N_FREQ, N2_FREQ, PLO, THE1, THE2, CF, YMI, YKI
  USE MOD_CONST     , ONLY : C43
  USE MOD_FUNC      , ONLY : FUNC_QP, FUNC_QS
  USE MOD_MODEL     , ONLY : MX, PTS, PTS2, H, H2, HPTS, XBMIN
  USE MOD_PARAMETERS, ONLY : MI, KI

!-----------------------------------------------------------------------

  IMPLICIT NONE

  INTEGER, INTENT(IN) :: LL

  INTEGER  :: K, II, KK, LAYER
  REAL(WP) :: XX, ZZ, QS, QP, THE1S, THE2S, THE1P, THE2P, R, TMP1, TMP2

  REAL(WP)   , DIMENSION( N_FREQ) :: YS, YP
  REAL(WP)   , DIMENSION(N2_FREQ) :: HE_QS, HE_QP
  COMPLEX(WP), DIMENSION(N2_FREQ) :: SUM1, SUM2

!-----------------------------------------------------------------------

  INTERFACE

    SUBROUTINE     QYLSM(Q,Y)
      USE MOD_PRECISION , ONLY : WP
      USE MOD_ATTEN     , ONLY : N_FREQ, N2_FREQ, FJK, FJJ
      IMPLICIT NONE
      REAL(WP), DIMENSION (N2_FREQ), INTENT( IN) :: Q
      REAL(WP), DIMENSION ( N_FREQ), INTENT(OUT) :: Y
    END SUBROUTINE QYLSM

    FUNCTION     GET_LAYER (XX,ZZ)
      USE MOD_PRECISION, ONLY : WP
      USE MOD_MODEL    , ONLY : INT_SED, NI
      IMPLICIT NONE
      INTEGER  :: GET_LAYER
      REAL(WP) :: XX, ZZ
    END FUNCTION GET_LAYER

  END INTERFACE

!----------------------------------------------------------------------

  IF ( LL == 0 ) THEN

    DO KK = 1+PTS, PTS2+PTS

      ZZ = -H2 + (KK-1)*HPTS

      DO II = 1, MX*(PTS2-1)+PTS+1
        XX    = XBMIN-H2 + (II-1)*HPTS
        LAYER = GET_LAYER(XX,ZZ)

        ! calculation of anelastic coefficient YS and YP
        QS = FUNC_QS (XX,ZZ,LAYER)
        QP = FUNC_QP (XX,ZZ,LAYER)
        DO K = 1, N2_FREQ
          HE_QS(K) = QS - PLO(K)
          HE_QP(K) = QP - PLO(K)
        END DO
        CALL QYLSM(HE_QS,YS)
        CALL QYLSM(HE_QP,YP)

        ! calculation of unrelaxed moduli MI and KI
        THE1S = 0._WP;    THE1P = 0._WP
        THE2S = 0._WP;    THE2P = 0._WP

        THE1S = 1 - SUM ( YS(:)*THE1(:) )
        THE2S =     SUM ( YS(:)*THE2(:) )
        THE1P = 1 - SUM ( YP(:)*THE1(:) )
        THE2P =     SUM ( YP(:)*THE2(:) )

        R = SQRT( THE1S*THE1S + THE2S*THE2S )
        TMP1 = 1/MI(II,KK) * (R+THE1S)/(2._WP*R*R)      ! mu

        R = SQRT( THE1P*THE1P + THE2P*THE2P )
        TMP2 = 1/KI(II,KK) + C43/MI(II,KK)              ! lambda+2mu
        TMP2 = TMP2 * (R+THE1P)/(2._WP*R*R)

        MI(II,KK) = 1 / TMP1                            ! 1/mu
        KI(II,KK) = 1 / (TMP2-C43*TMP1)                 ! 1/kappa

        ! calculation of frequency dependent (complex) moduli mu and
        ! kappa, stored in fields YMI and YKI until integration by INT2D_Y
        SUM1 = CMPLX(0.,0.)
        SUM2 = CMPLX(0.,0.)
        DO K = 1, N_FREQ
          SUM1(:) = SUM1(:) + YS(K)*CF(K,:)
          SUM2(:) = SUM2(:) + YP(K)*CF(K,:)
        END DO
        SUM1(:) = 1 - SUM1(:)
        SUM2(:) = 1 - SUM2(:)
        YMI(II,KK,:) = 1 / ( TMP1*SUM1(:)                    )  ! 1/mu(omega)
        YKI(II,KK,:) = 1 / ( TMP2*SUM2(:) - C43*TMP1*SUM1(:) )  ! 1/kappa(omega)
      END DO

    END DO

    MI (:,1:PTS) = MI (:,PTS2:1+PTS:-1)
    KI (:,1:PTS) = KI (:,PTS2:1+PTS:-1)

    YMI(:,1:PTS,:) = YMI(:,PTS2:1+PTS:-1,:)
    YKI(:,1:PTS,:) = YKI(:,PTS2:1+PTS:-1,:)

  ELSE

    DO KK = 1, PTS2+PTS

      ZZ = LL*H-H2 + (KK-1)*HPTS

      DO II = 1, MX*(PTS2-1)+PTS+1
        XX    = XBMIN-H2 + (II-1)*HPTS
        LAYER = GET_LAYER(XX,ZZ)

        ! calculation of anelastic coefficient YS and YP
        QS = FUNC_QS (XX,ZZ,LAYER)
        QP = FUNC_QP (XX,ZZ,LAYER)
        DO K = 1, N2_FREQ
          HE_QS(K) = QS - PLO(K)
          HE_QP(K) = QP - PLO(K)
        END DO
        CALL QYLSM(HE_QS,YS)
        CALL QYLSM(HE_QP,YP)

        ! calculation of unrelaxed moduli MUI and KI
        THE1S = 0._WP;    THE1P = 0._WP
        THE2S = 0._WP;    THE2P = 0._WP

        THE1S = 1 - SUM ( YS(:)*THE1(:) )
        THE2S =     SUM ( YS(:)*THE2(:) )
        THE1P = 1 - SUM ( YP(:)*THE1(:) )
        THE2P =     SUM ( YP(:)*THE2(:) )

        R = SQRT( THE1S*THE1S + THE2S*THE2S )
        TMP1 = 1/MI(II,KK) * (R+THE1S)/(2._WP*R*R)      ! mu

        R = SQRT( THE1P*THE1P + THE2P*THE2P )
        TMP2 = 1/KI(II,KK) + C43/MI(II,KK)              ! lambda+2mu
        TMP2 = TMP2 * (R+THE1P)/(2._WP*R*R)

        MI(II,KK) = 1 / TMP1                            ! 1/mu
        KI(II,KK) = 1 / (TMP2-C43*TMP1)                 ! 1/kappa

        ! calculation of frequency dependent (complex) moduli mu and
        ! kappa, stored in fields YMI and YKI until integration by INT2D_Y
        SUM1 = CMPLX(0.,0.)
        SUM2 = CMPLX(0.,0.)
        DO K = 1, N_FREQ
          SUM1(:) = SUM1(:) + YS(K)*CF(K,:)
          SUM2(:) = SUM2(:) + YP(K)*CF(K,:)
        END DO
        SUM1(:) = 1 - SUM1(:)
        SUM2(:) = 1 - SUM2(:)
        YMI(II,KK,:) = 1 / ( TMP1*SUM1(:)                    )  ! 1/mu(omega)
        YKI(II,KK,:) = 1 / ( TMP2*SUM2(:) - C43*TMP1*SUM1(:) )  ! 1/kappa(omega)

      END DO

    END DO

  END IF


END SUBROUTINE PREPARE_INTEG_Y
